import logging
import os

from notetool.secret import read_secret
from sqlalchemy import MetaData, create_engine
from sqlalchemy.orm import declarative_base, sessionmaker

uri = read_secret(cate1='notecoin', cate2='dataset', cate3='db_path')
uri = uri or f'sqlite:///{os.path.abspath(os.path.dirname(__file__))}/data/notecoin.db'
# engine = create_engine(uri, echo=True)
meta = MetaData()
engine = create_engine(uri)
Base = declarative_base()

logging.info(f'uri:{uri}')


def create_session():
    # return scoped_session(sessionmaker(autocommit=True,autoflush=True,bind=engine))
    # return sessionmaker(autocommit=True, autoflush=True, bind=engine)()
    return sessionmaker(bind=engine)()


def create_all():
    Base.metadata.create_all(engine)
    session.commit()


class BaseTable(object):
    def __init__(self, *args, **kwargs):
        self.session = create_session()

    def json(self):
        res = {k: v for k, v in self.__dict__.items() if not k.startswith("_")}
        return res


session = create_session()



import csv
import json
import logging
import time
from datetime import datetime

import ccxt
import pandas as pd
from ccxt.base.exchange import Exchange
from notecoin.base.database import TradeData
from tqdm import tqdm

logger = logging.getLogger()
unix_month = 2678400000
one_hour = 3600 * 1000


class LoadData:
    def __init__(self, exchange: Exchange, csv_path, unix_start, unix_end, *args, **kwargs):
        self.exchange = exchange
        self.csv_path = csv_path
        self.unix_start = unix_start
        self.unix_end = unix_end
        self.cache_data = []

    def load_open(self):
        pass

    def load_all(self, *args, **kwargs):
        self.load_open()
        self.exchange.load_markets()
        pbr = tqdm(self.exchange.symbols)
        for sym in pbr:
            if ':' not in sym:
                pbr.set_description(sym)
                self.load(sym, *args, **kwargs)
        self.write_data([], False)
        self.load_close()

    def load_close(self):
        pass

    def load(self, symbol, *args, **kwargs):
        pass

    def write_data(self, data_list, cache=True):
        self.cache_data.extend(data_list)
        if cache and len(self.cache_data) < 30000:
            return
        if len(self.cache_data) == 0:
            return
        df = pd.DataFrame(self.cache_data)
        df = df[(df['timestamp'] >= self.unix_start) & (df['timestamp'] <= self.unix_end)]
        self._write(json.loads(df.to_json(orient='records')))

    def _write(self, data_list):
        pass


class LoadDataKline(LoadData):
    def __init__(self, *args, timeframe='1m', **kwargs):
        super(LoadDataKline, self).__init__(*args, **kwargs)
        self.timeframe = timeframe
        self.csv_writer = csv.DictWriter(open(self.csv_path, mode="w"), delimiter=",",
                                         fieldnames=["symbol", "timestamp", "open", "close", "low", "high", "vol"])

    def _write(self, data_list):
        self.csv_writer.writerows(data_list)

    def load(self, symbol, *args, **kwargs):
        unix_temp = self.unix_start
        while unix_temp < self.unix_end:
            try:
                result = self.exchange.fetch_ohlcv(symbol, self.timeframe, unix_temp, limit=100)
                result = self.exchange.sort_by(result, 0)
                if len(result) == 0:
                    break
                unix_temp = result[-1][0]
                df = pd.DataFrame(result, columns=['timestamp', 'open', 'close', 'low', 'high', 'vol'])
                df['symbol'] = symbol
                self.write_data(json.loads(df.to_json(orient='records')))
                time.sleep(int(self.exchange.rateLimit / 1000))
            except Exception as e:
                print(type(e).__name__, str(e))
                self.exchange.sleep(10000)


class LoadTradeKline(LoadData):
    def __init__(self, *args, **kwargs):
        super(LoadTradeKline, self).__init__(*args, **kwargs)
        # self.csv_writer = csv.DictWriter(open(self.csv_path, mode="w"), delimiter=",",
        #                                  fieldnames=["symbol", "id", "timestamp", "type", "side", "price", "amount"])
        self.table = TradeData(
            table_name=f"{self.exchange.name}-trade-{datetime.utcfromtimestamp(self.unix_start).strftime('%Y%m%d')}")

    def load_close(self):
        self.table.to_csv_all(file_path=self.csv_path)

    def _write(self, data_list):
        self.table.insert(values=data_list)
        # self.csv_writer.writerows(data_list)

    def load(self, symbol, *args, **kwargs):
        unix_temp = self.unix_start
        previous_trade_id = None

        while unix_temp < self.unix_end:
            try:
                trades = self.exchange.fetch_trades(symbol, unix_temp, limit=1000)
                if len(trades) == 0:
                    unix_temp += one_hour
                    continue
                last_trade = trades[-1]
                if previous_trade_id == last_trade['id']:
                    unix_temp += one_hour
                    continue
                unix_temp = last_trade['timestamp']
                previous_trade_id = last_trade['id']

                result = [{
                    'symbol': trade['symbol'],
                    'id': trade['id'].replace('\n', ''),
                    'timestamp': trade['timestamp'],
                    'type': trade['type'],
                    'side': trade['side'],
                    'price': trade['price'],
                    'amount': trade['amount'],
                } for trade in trades]

                self.write_data(result)
                time.sleep(int(self.exchange.rateLimit / 1000))
            except ccxt.NetworkError as e:
                print(type(e).__name__, str(e))
                self.exchange.sleep(10000)



import logging

from ccxt import binance, okex
from notecoin.coins.base.file import DataFileProperty

logger = logging.getLogger()
logger.setLevel(logging.INFO)

path_root = '/home/bingtao/workspace/tmp'


def load_okex():
    file_pro = DataFileProperty(exchange=okex(), path=path_root)
    file_pro.change_data_type('trade')
    file_pro.change_timeframe('detail')
    file_pro.change_freq('daily')
    file_pro.load(total=1)
    # file_pro.change_freq('weekly')
    # file_pro.load(total=60)
    # file_pro.change_freq('monthly')
    # file_pro.load(total=12)

    file_pro.change_data_type('kline')
    file_pro.change_timeframe('1m')
    file_pro.change_freq('daily')
    file_pro.load(total=450)
    file_pro.change_freq('weekly')
    file_pro.load(total=60)


def load_binance():
    file_pro = DataFileProperty(exchange=binance(), path=path_root)
    file_pro.change_data_type('trade')
    file_pro.change_timeframe('detail')
    file_pro.change_freq('daily')
    file_pro.load(total=450)
    file_pro.change_freq('weekly')
    file_pro.load(total=60)
    # file_pro.change_freq('monthly')
    # file_pro.load(total=12)

    file_pro.change_data_type('kline')
    file_pro.change_timeframe('1m')
    file_pro.change_freq('daily')
    file_pro.load(total=450)
    file_pro.change_freq('weekly')
    file_pro.load(total=60)


# load_binance()
load_okex()



import time

from notecoin.database.base import Base, BaseTable
from notecoin.strategy.utils import trade_api
from sqlalchemy import Column, Float, String


class OkexStrategyAutoSeller(Base, BaseTable):
    __tablename__ = 'okex_strategy_auto_seller'
    coin_id = Column(String(20), comment='coin_id', primary_key=True)
    price = Column(Float, comment='price')
    count = Column(Float, comment='count')
    worth = Column(Float, comment='worth')
    init_worth = Column(Float, comment='init_worth')
    max_worth = Column(Float, comment='max_worth')
    min_worth = Column(Float, comment='min_worth')
    updateTime = Column(Float, comment='updateTime')

    def __init__(self, *args, **kwargs):
        self.coin_id = kwargs.get("coin_id")
        self.price = kwargs.get("price")
        self.count = kwargs.get("count")
        self.worth = kwargs.get("worth")
        self.init_worth = kwargs.get("init_worth") or kwargs.get("worth")
        self.max_worth = kwargs.get("max_worth") or kwargs.get("init_worth") or kwargs.get("worth")
        self.min_worth = kwargs.get("min_worth") or kwargs.get("init_worth") or kwargs.get("worth")
        self.updateTime = int(time.time() * 1000)
        super(OkexStrategyAutoSeller, self).__init__(*args, **kwargs)

    def update_worth(self, worth, price=None, count=None):
        self.worth = worth
        self.price = price or self.price
        self.count = count or self.count
        if worth > self.max_worth:
            self.max_worth = worth
        if worth < self.min_worth:
            self.min_worth = worth
        self.updateTime = int(time.time() * 1000)

    def check(self):
        if self.max_worth * 0.95 > self.worth > self.init_worth * 1.01:
            return True
        else:
            return False

    def to_json(self):
        return {
            "coin_id": self.coin_id,
            "usdt": round(self.worth, 2)
        }

    def buy(self, worth=50):
        return trade_api.place_order(instId=self.coin_id, tdMode='cash', side='buy', ordType='market', sz=worth)

    def sell(self):
        return trade_api.place_order(instId=self.coin_id, tdMode='cash', side='sell', ordType='market', sz=self.count)

    @staticmethod
    def instance(coin_id, price=None, count=None, worth=None, init_worth=50):
        return OkexStrategyAutoSeller(coin_id=coin_id, price=price, count=count, worth=worth, init_worth=init_worth)
